#region Copyright Syncfusion Inc. 2001 - 2016
// Copyright Syncfusion Inc. 2001 - 2016. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion

using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Input;
using FilteringDemo.Resources;

namespace FilteringDemo.ViewModel
{
    class EmployeeInfoViewModel : ViewModelBase
    {        

        #region Properties

        private string _filterOption = "All Columns";
        public string FilterOption
        {
            get { return _filterOption; }
            set
            {
                _filterOption = value;
                NotifyPropertyChanged("FilterOption");
            }
        }

        private string _filterText = string.Empty;
        public string FilterText
        {
            get { return _filterText; }
            set
            {
                _filterText = value;
                NotifyPropertyChanged("FilterText");
            }
        }

        private string _filterCondition;
        public string FilterCondition
        {
            get { return _filterCondition; }
            set
            {
                _filterCondition = value;
                NotifyPropertyChanged("FilterCondition");
            }
        }

        private ObservableCollection<Employees> _employeeDetails;
        public ObservableCollection<Employees> EmployeeDetails
        {
            get
            {
                return _employeeDetails;
            }
            set
            {
                _employeeDetails = value;
                NotifyPropertyChanged("EmployeeDetails");
            }
        }

        #endregion

        internal FilterChanged FilterChanged;
                
        // Comandos
        public ICommand TextChanged { get; set; }
        public ICommand ComboChanged { get; set; }
        public ICommand FilterComboChanged { get; set; }


        #region Constructor

        public EmployeeInfoViewModel()
        {
            EmployeeDetails = new EmployeeInfoRespository().GetEmployeesDetails(2000);

            TextChanged = new RelayCommand<object>(TextChangeEvent);
            ComboChanged = new RelayCommand<object>(ComboxChangedEvent);
            FilterComboChanged = new RelayCommand<object>(FilterComboxChangedEvent);
        }

        #endregion

        #region Methods

        private void FilterComboxChangedEvent(object pram)
        {
            if (pram != null)
            {
                FilterOption = (pram as ComboBoxItem).Content.ToString();
                FilterChanged();
            }
        }

        private void ComboxChangedEvent(object pram)
        {
            if (pram != null)
            {
                FilterCondition = (pram as ComboBoxItem).Content.ToString();
                FilterChanged();
            }
        }

        private void TextChangeEvent(object pram)
        {
            if (pram != null)
            {
                FilterText = pram.ToString();
                FilterChanged();
            }
        }

        private bool MakeStringFilter(Employees o, string option, string condition)
        {
            var value = o.GetType().GetProperty(option);
            var exactValue = value.GetValue(o,null);
            exactValue = exactValue.ToString().ToLower();

            var text = FilterText.ToLower();
            var methods = typeof(string).GetMethods();

            if (methods.Length != 0)
            {
                var methodInfo = methods.FirstOrDefault(method => method.Name == condition);
                bool result1 = (bool)methodInfo.Invoke(exactValue, new object[] { text });

                return result1;
            }

            return false;
        }

        private bool MakeNumericFilter(Employees o, string option, string condition)
        {
            var value = o.GetType().GetProperty(option);
            var exactValue = value.GetValue(o,null);
            double res;
            bool checkNumeric = double.TryParse(exactValue.ToString(), out res);

            if (checkNumeric)
            {
                switch (condition)
                {
                    case "Equals":
                        if (Convert.ToDouble(exactValue) == (Convert.ToDouble(FilterText)))
                            return true;
                        break;

                    case "GreaterThan":
                        if (Convert.ToDouble(exactValue) > Convert.ToDouble(FilterText))
                            return true;
                        break;

                    case "LessThan":
                        if (Convert.ToDouble(exactValue) < Convert.ToDouble(FilterText))
                            return true;
                        break;

                    case "NotEquals":
                        if (Convert.ToDouble(FilterText) != Convert.ToDouble(exactValue))
                            return true;
                        break;
                }
            }

            return false;
        }

        public bool FilerRecords(object o)
        {
            double res;
            var checkNumeric = double.TryParse(FilterText, out res);
            var item = o as Employees;

            if (item != null && FilterText.Equals(""))            
                return true;
            
            
            if (item != null)
            {
                if (checkNumeric && !FilterOption.Equals("All Columns"))
                {
                    if (FilterCondition == null || FilterCondition.Equals("Contains") || FilterCondition.Equals("StartsWith") || FilterCondition.Equals("EndsWith"))
                        FilterCondition = "Equals";

                    bool result = MakeNumericFilter(item, FilterOption, FilterCondition);

                    return result;
                }

                if (FilterOption.Equals("All Columns"))
                {
                    if (item.Name.ToLower().Contains(FilterText.ToLower()) ||
                        item.Title.ToLower().Contains(FilterText.ToLower()) ||
                        item.Salary.ToString().ToLower().Contains(FilterText.ToLower()) || item.EmployeeID.ToString().ToLower().Contains(FilterText.ToLower()) ||
                        item.Gender.ToLower().Contains(FilterText.ToLower()) )
                        return true;
                    return false;
                }
                else
                {
                    if (FilterCondition == null || FilterCondition.Equals("Equals") || FilterCondition.Equals("LessThan") || FilterCondition.Equals("GreaterThan") || FilterCondition.Equals("NotEquals"))
                       FilterCondition = "Contains";
                    bool result = MakeStringFilter(item, FilterOption, FilterCondition);
                    return result;
                }
            }
            
            return false;
        }

        #endregion
    }

    internal delegate void FilterChanged();
}
